/* decode.c
 * Part of the RTAudio.Client project.
 * This file contains the generic routine for reading in compressed sound
 * from one stream, decompressing it, and writing it to another stream.
 * Written by Electronic One <dkaye@cybersp.mb.ca>
 */

/* Lookup table */
#include "short2char.h"
#include "errorcodes.h"
#include "exit.h"
#include "switches.h"
#include "curses.h"
#include "modes.h"

#include <stdio.h>

/* The ultra-elite sound compressor */
#include "gsm.h"

void decode_sound(SW *sw, FILE *inf, FILE *outf)
{
   gsm handle;
   unsigned char buf[320];  /* Dual-use buffer */
   short sample[160*2];
   int errcode = 0, i;
      
   if((handle = gsm_create()) == 0)
     {
	p_perror(sw->mode, "Error creating sound handle");
	return;
     }

   if(sw->mode == MODE_TEXT)
     initscr();
 
   cbreak();
   keypad(stdscr, TRUE);
   nodelay(stdscr, TRUE);
   
   while(!errcode)
     {
        if(fread(&buf[0], 33, 1, inf) != 1)
          {
             p_perror(sw->mode, "Read error");
	     errcode = ERR_READ;
          }
	
        /* Decompress it */
        if(gsm_decode(handle, &buf[0], &sample[0]))
          p_perror(sw->mode, "Received invalid frame");
	
        /* Convert it */
        for(i = 0; i < 160; ++i)
          *(&buf[0] + i) = s2c(*(&sample[0] + i));
	
        /* Play it back */
        if(fwrite(&buf[0], 1, 160, outf) != 160)
          {
             p_perror(sw->mode, "Error writing to output");
	     errcode = ERR_WRITE;
          }
	
	if(getch() != ERR)
	  errcode = ERR_UNKNOWN;
     }
   
   gsm_destroy(handle);

   if(sw->mode == MODE_TEXT)
     {
	endwin();
     }
}


